import { Component, OnInit } from '@angular/core';
import { commonReducers, } from '@farmmaps/common';
import {  } from '@farmmaps/common-map';
import {mapReducers,mapActions,IItemLayer} from '@farmmaps/common-map';
import { Store } from '@ngrx/store';
import { tassign } from 'tassign';
import { Observable } from 'rxjs';
import {createEmpty,extend } from 'ol/extent';

@Component({
  selector: 'app-menu',
  templateUrl: './menu.component.html',
  styleUrls: ['./menu.component.scss']
})
export class MenuComponent implements OnInit {
  public baseLayersCollapsed:boolean = true;
  public overlayLayersCollapsed: boolean = true;
  public overlayLayers: Observable<Array<IItemLayer>>;
  public selectedOverlayLayer: Observable<IItemLayer>;
  public baseLayers: Observable<Array<IItemLayer>>;
  public selectedBaseLayer: Observable<IItemLayer>;
  

  constructor( private store: Store<mapReducers.State | commonReducers.State> ) { }

  ngOnInit() {
    this.overlayLayers = this.store.select(mapReducers.selectGetOverlayLayers);
    this.selectedOverlayLayer = this.store.select(mapReducers.selectGetSelectedOverlayLayer);
    this.baseLayers = this.store.select(mapReducers.selectGetBaseLayers);
    this.selectedBaseLayer = this.store.select(mapReducers.selectGetSelectedBaseLayer);
  }

  handleToggleBaseLayers(event:MouseEvent) {
    this.baseLayersCollapsed = !this.baseLayersCollapsed;
    event.preventDefault();
  }

  handleToggleOverlayLayers(event: MouseEvent) {
    this.overlayLayersCollapsed = !this.overlayLayersCollapsed;
    event.preventDefault();
  }

  handlePredefinedQuery(event: MouseEvent, query: any) {
    event.preventDefault();
    var queryState = tassign(mapReducers.initialQueryState, query);
    this.store.dispatch(new mapActions.DoQuery(queryState));
  }

  handleOnToggleVisibility(itemLayer: IItemLayer) {
    this.store.dispatch(new mapActions.SetVisibility(itemLayer,!itemLayer.visible));
  }

  handleOnSetOpacity(event:{ layer: IItemLayer,opacity:number }) {
    this.store.dispatch(new mapActions.SetOpacity(event.layer, event.opacity));
  }

  handleOnDelete(itemLayer: IItemLayer) {
    this.store.dispatch(new mapActions.RemoveLayer(itemLayer));
  }

  handleZoomToExtent(itemLayer: IItemLayer) {
    var extent =  createEmpty();
    extend(extent, itemLayer.layer.getExtent());
    if (extent) {
      this.store.dispatch(new mapActions.SetExtent(extent));
    }    
  }

  handleSelectOverlayLayer(itemLayer: IItemLayer) {
    this.store.dispatch(new mapActions.SelectOverlayLayer(itemLayer));
  }

  handleSelectBaseLayer(itemLayer: IItemLayer) {
    this.store.dispatch(new mapActions.SelectBaseLayer(itemLayer));
  }

}
